// SoapRelayServiceHost.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Configuration;
using System.ComponentModel;
using System.Data;
using System.Text;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Description;
using System.Runtime.Serialization;

namespace Microsoft.Presto.StarterKit.NonNormative
{
    public class SoapRelayServiceHost
    {
        private ServiceHost serviceHost;
        MyRouter router = new MyRouter();

        public bool IsListening
        {
            get { return (bool)(serviceHost != null); }
        }

        internal void StartSoapRelayInterface()
        {
            if (serviceHost != null) StopSoapRelayInterface();

            WSHttpBinding binding = new WSHttpBinding();
            binding.Security.Mode = SecurityMode.None;

            // Create a service host
            Uri baseAddress = new Uri(ConfigurationManager.AppSettings["baseAddress"]);
            serviceHost = new ServiceHost(router, baseAddress);

            // Add an endpoint
            serviceHost.AddServiceEndpoint(typeof(ISoapRelay), binding, "http://soaprelay");
            ServiceEndpoint serviceEndpoint = serviceHost.Description.Endpoints.Find(new Uri("http://soaprelay"));
            //MatchAllEndpointBehavior maepb = new MatchAllEndpointBehavior();
            //serviceEndpoint.Behaviors.Add(maepb);
            serviceEndpoint.ListenUri = new Uri(ConfigurationManager.AppSettings["baseAddress"] + "/soaprelay");

            // Create and open the service runtime
            serviceHost.Open();
            DisplayServiceEndpointsState();
        }

        public void StopSoapRelayInterface()
        {
            if (serviceHost != null)
            {
                serviceHost.Close();
                DisplayServiceEndpointsState();
                serviceHost = null;
            }
        }

        private void DisplayServiceEndpointsState()
        {
            // Display the Service endpoints state
            OutputTrace.Log("Soap Relay Endpoints:");
            foreach (ChannelDispatcher channelDispatcher in serviceHost.ChannelDispatchers)
            {
                foreach (EndpointDispatcher endpointDispatcher in channelDispatcher.Endpoints)
                {
                    StringBuilder sb = new StringBuilder();
                    sb.AppendFormat("Listener ({0}, {1}) = {2}, State = {3}",
                            endpointDispatcher.ContractName.ToString(),
                            endpointDispatcher.ContractNamespace.ToString(),
                            endpointDispatcher.EndpointAddress.ToString(),
                            channelDispatcher.State.ToString());
                    OutputTrace.Log(sb.ToString());
                }
            }
            OutputTrace.Log(Environment.NewLine);
        }
    }

    //[SoapRelayServiceBehavior(FinalDestination="net.tcp://localhost:8080/proxy")]
    [SoapRelayServiceBehavior(FinalDestination = "net.pipe://localhost/proxy")]
    [ServiceBehavior(InstanceContextMode = InstanceContextMode.Single)]
    class MyRouter : ISoapRelay
    {
        private SoapRelayExtension SoapRelayExtension = null;

        public MyRouter()
        {

        }

        #region ISoapRelay Members

        System.ServiceModel.Channels.Message ISoapRelay.ProcessMessage(System.ServiceModel.Channels.Message message)
        {
            if (SoapRelayExtension == null)
            {
                ServiceHostBase host = OperationContext.Current.Host;
                this.SoapRelayExtension = host.Extensions.Find<SoapRelayExtension>();
            }

            StringBuilder sb = new StringBuilder();
            sb.AppendFormat("Routing message to {0}", SoapRelayExtension.FinalDestination);
            OutputTrace.Log(sb.ToString());

            return (SoapRelayExtension.ForwardingChannel.ProcessMessage(message));
        }

        #endregion
    }
}