// TraceBehavior.cs
//
// PRESTO Starter Kit for Microsoft .NET 3.0
// 2007 Microsoft France
// Published under the CECILL-B Free Software license agreement.
// (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
// 
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
// WHETHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
// WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE. 
// THE ENTIRE RISK OF USE OR RESULTS IN CONNECTION WITH THE USE OF THIS CODE 
// AND INFORMATION REMAINS WITH THE USER. 
//

using System;
using System.Collections.Generic;
using System.Text;
using System.ServiceModel.Dispatcher;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Xml;
using System.ServiceModel.Description;
using System.IO;

namespace Microsoft.Presto.StarterKit
{
    public class TraceBehavior : IDispatchMessageInspector, IErrorHandler
    {

        #region IDispatchMessageInspector Members

        public object AfterReceiveRequest(ref Message request, IClientChannel channel, InstanceContext instanceContext)
        {
            OutputTrace.Log("----------------------------------------------------------------------------------------------");
            OutputTrace.Log("AfterReceiveRequest");

            StringBuilder sbuilder = new StringBuilder();
            OperationContext ctx = OperationContext.Current;
     
            //HttpRequestMessageProperty prop = ctx.IncomingMessageProperties[HttpRequestMessageProperty.Name] as HttpRequestMessageProperty;
            //foreach (string info in prop.Headers.Keys)
            //    Logger.Log(info + ":" + prop.Headers[info]);
            //Logger.Log("");
            OutputTrace.Log(request.ToString());

            return 0;
        }

        public void BeforeSendReply(ref Message reply, object correlationState)
        {
            OutputTrace.Log("----------------------------------------------------------------------------------------------");
            OutputTrace.Log("BeforeSendReply");
            if (reply != null) OutputTrace.Log(reply.ToString());
        }
        #endregion

        #region IErrorHandler Members
        public bool HandleError(Exception error)
        {
            OutputTrace.Log("HandleError");
            if (error != null)
            {
                OutputTrace.Log("Error Message : " + error.Message);
                try
                {
                    OutputTrace.Log(OperationContext.Current.RequestContext.RequestMessage.ToString());
                }
                catch (Exception ex) 
                {
                    OutputTrace.Log(ex.Message);
                }
            }
            return true;
        }

        public void ProvideFault(Exception error, MessageVersion version, ref Message fault)
        {
            OutputTrace.Log("ProvideFault");
        }
        #endregion

    }

    public class TraceBehaviorAttribute : Attribute, IServiceBehavior
    {
        TraceBehavior traceBehavior = new TraceBehavior();

        public TraceBehaviorAttribute()
        {
        }

        void IServiceBehavior.Validate(ServiceDescription description, ServiceHostBase serviceHostBase)
        {
        }

        void IServiceBehavior.AddBindingParameters(ServiceDescription description, ServiceHostBase serviceHostBase, System.Collections.ObjectModel.Collection<ServiceEndpoint> endpoints, BindingParameterCollection parameters)
        {
        }

        void IServiceBehavior.ApplyDispatchBehavior(ServiceDescription description, ServiceHostBase serviceHostBase)
        {                                      
            foreach (ChannelDispatcherBase channelDispatcherBase in serviceHostBase.ChannelDispatchers)
            {
                
                ChannelDispatcher channelDispatcher = channelDispatcherBase as ChannelDispatcher;
                channelDispatcher.ErrorHandlers.Add(traceBehavior);                   
                channelDispatcher.Listener.Faulted += new EventHandler(Listener_Faulted);
                channelDispatcher.IncludeExceptionDetailInFaults = true;
                EndpointDispatcher edispatcher = channelDispatcher.Endpoints[0];
  

                for (int j = 0; j < channelDispatcher.Endpoints.Count; ++j)
                {
                    EndpointDispatcher endpointDispatcher = channelDispatcher.Endpoints[j];
                    endpointDispatcher.DispatchRuntime.MessageInspectors.Add(traceBehavior);
                    foreach (DispatchOperation dispatchOp in endpointDispatcher.DispatchRuntime.Operations)
                    {
                        // OneWay and Request operations
                    }
                    //endpointDispatcher.DispatchRuntime.UnhandledDispatchOperation.ParameterInspectors.Add(new CustomParamInspector());
                }
            }
        }

        void Listener_Faulted(object sender, EventArgs e)
        {
            OutputTrace.Log("Listener faulted");
        }
    }
}

